@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/python2
 # -*- coding: UTF-8 -*-
 
 ############################################################################
@@ -50,12 +50,7 @@ gettext.textdomain('nemo-terminal')
 _ = gettext.gettext
 
 import gi
-try:
-    gi.require_version('Vte', '2.90')
-    VTE_2_90_API = True
-except:
-    gi.require_version('Vte', '2.91')
-    VTE_2_90_API = False
+gi.require_version('Vte', '2.91')
 gi.require_version('Nemo', '3.0')
 from gi.repository import GObject, Nemo, Gtk, Gdk, Vte, GLib, Gio
 
@@ -83,7 +78,7 @@ class NemoTerminal(object):
         uri -- The URI of the folder where the terminal will be created.
         window -- The parent window.
     """
-    
+
     def __init__(self, uri, window):
         """The constructor."""
         self._window = window
@@ -92,73 +87,68 @@ class NemoTerminal(object):
         self.shell_pid = -1
         self.term = Vte.Terminal()
 
-        if VTE_2_90_API:
-            self.shell_pid = self.term.fork_command_full(Vte.PtyFlags.DEFAULT, self._path, [terminal_or_default()], None, GLib.SpawnFlags.SEARCH_PATH, None, None)[1]
-        else:
-            self.shell_pid = self.term.spawn_sync(Vte.PtyFlags.DEFAULT, self._path, [terminal_or_default()], None, GLib.SpawnFlags.SEARCH_PATH, None, None, None)[1]
+        self.shell_pid = self.term.spawn_sync(Vte.PtyFlags.DEFAULT, self._path, [terminal_or_default()], None, GLib.SpawnFlags.SEARCH_PATH, None, None, None)[1]
+
         # Make vte.sh active
         #vte_current_dir_script = ". /etc/profile.d/vte.sh ; clear"
-        #self.term.feed_child(vte_current_dir_script, len(vte_current_dir_script))     
-        
+        #self.term.feed_child(vte_current_dir_script, len(vte_current_dir_script))
+
         self.term.connect_after("child-exited", self._on_term_child_exited)
         self.term.connect_after("popup-menu", self._on_term_popup_menu)
-        self.term.connect("button-release-event", self._on_term_popup_menu)
-        
+        self.term.connect("button-press-event", self._on_term_popup_menu)
+
         #Accelerators
         accel_group = Gtk.AccelGroup()
         self._window.add_accel_group(accel_group)
         self.term.add_accelerator(
-                "paste-clipboard",
-                accel_group,
-                ord("V"),
-                Gdk.ModifierType.CONTROL_MASK | Gdk.ModifierType.SHIFT_MASK,
-                Gtk.AccelFlags.VISIBLE)
+            "paste-clipboard",
+            accel_group,
+            ord("V"),
+            Gdk.ModifierType.CONTROL_MASK | Gdk.ModifierType.SHIFT_MASK,
+            Gtk.AccelFlags.VISIBLE)
         self.term.add_accelerator(
-                "copy-clipboard",
-                accel_group,
-                ord("C"),
-                Gdk.ModifierType.CONTROL_MASK | Gdk.ModifierType.SHIFT_MASK,
-                Gtk.AccelFlags.VISIBLE)
+            "copy-clipboard",
+            accel_group,
+            ord("C"),
+            Gdk.ModifierType.CONTROL_MASK | Gdk.ModifierType.SHIFT_MASK,
+            Gtk.AccelFlags.VISIBLE)
         #Drag & Drop
         self.term.drag_dest_set(
-                Gtk.DestDefaults.MOTION |
-                Gtk.DestDefaults.HIGHLIGHT |
-                Gtk.DestDefaults.DROP,
-                [Gtk.TargetEntry.new("text/uri-list", 0, 80)],
-                Gdk.DragAction.COPY,
-                )
+            Gtk.DestDefaults.MOTION |
+            Gtk.DestDefaults.HIGHLIGHT |
+            Gtk.DestDefaults.DROP,
+            [Gtk.TargetEntry.new("text/uri-list", 0, 80)],
+            Gdk.DragAction.COPY,
+        )
         self.term.drag_dest_add_uri_targets()
         self.term.connect("drag_data_received", self._on_drag_data_received)
-        
+
         # Container
-        if VTE_2_90_API:
-            self.vscrollbar = Gtk.VScrollbar.new(self.term.adjustment)
-        else:
-            self.vscrollbar = Gtk.VScrollbar.new(self.term.get_vadjustment())
-        
+        self.vscrollbar = Gtk.VScrollbar.new(self.term.get_vadjustment())
+
         self.hbox = Gtk.HBox()
         self.hbox.pack_start(self.term, True, True, 0)
         self.hbox.pack_end(self.vscrollbar, False, False, 0)
-        
+
         self.hbox.nt = self # store a reference to this obj
-        
+
         #Popup Menu
         self.menu = Gtk.Menu()
         #MenuItem => copy
         menu_item = Gtk.ImageMenuItem.new_from_stock("gtk-copy", None)
         menu_item.connect_after("activate",
-                lambda w: self.term.copy_clipboard())
+                                lambda w: self.term.copy_clipboard())
         self.menu.add(menu_item)
         #MenuItem => paste
         menu_item = Gtk.ImageMenuItem.new_from_stock("gtk-paste", None)
         menu_item.connect_after("activate",
-                lambda w: self.term.paste_clipboard())
+                                lambda w: self.term.paste_clipboard())
         self.menu.add(menu_item)
         #MenuItem => paste filenames (only added if URI types on clipboard)
         menu_item_pastefilenames = \
             Gtk.ImageMenuItem.new_from_stock("gtk-paste", None)
         menu_item_pastefilenames.connect_after("activate",
-                lambda w: self._paste_filenames_clipboard())
+                                               lambda w: self._paste_filenames_clipboard())
         menu_item_pastefilenames.set_label(_("Paste Filenames"))
         self.menu_item_pastefilenames = menu_item_pastefilenames
         self.menu.add(menu_item_pastefilenames)
@@ -182,14 +172,13 @@ class NemoTerminal(object):
         #MenuItem => About
         menu_item = Gtk.ImageMenuItem.new_from_stock("gtk-about", None)
         menu_item.connect_after("activate",
-                lambda w: self.show_about_dialog())
+                                lambda w: self.show_about_dialog())
         self.menu.add(menu_item)
-        #
         self.menu.show_all()
-        #Conf
-        self._set_term_height( \
-            settings.get_value("default-terminal-height").get_byte())
+
+        self._set_term_height(settings.get_int("default-terminal-height"))
         self._visible = True
+
         #Lock
         self._respawn_lock = False
         #Register the callback for show/hide
@@ -199,7 +188,7 @@ class NemoTerminal(object):
     def _goto_current_terminal_directory(self):
         """Navigate the active Nemo pane to the current working directory
         of the VTE
-        
+
         This functionality depends on some changes to Nemo's extension
         interface.
         """
@@ -227,11 +216,13 @@ class NemoTerminal(object):
             uris = gtkClipboard.wait_for_uris()
             concatfilenames = ""
             for idx, uri in enumerate(uris):
-                path = Gio.file_parse_name(uri).get_path()
+                path = self._uri_to_path(uri)
+                if path == "":
+                    continue
                 quoted = GLib.shell_quote(path)
-                self.term.feed_child(quoted, len(quoted))
+                self.feed_child(quoted)
                 if idx != (len(uris)-1):
-                    self.term.feed_child(' ', 1)    
+                    self.feed_child(' ')
         return
 
     def change_directory(self, uri):
@@ -240,21 +231,29 @@ class NemoTerminal(object):
         Args:
             uri -- The URI of the destination directory.
         """
+
+        if settings.get_enum("default-follow-mode") in (0, 2):
+            return 
+
         self._path = self._uri_to_path(uri)
+
+        if self._path == "":
+            return
+
         if not self._shell_is_busy():
             # Clear any input
             eraselinekeys = settings.get_string("terminal-erase-line").decode('string_escape')
-            self.term.feed_child(eraselinekeys, len(eraselinekeys))
-            
+            self.feed_child(eraselinekeys)
+
             # Change directory
             cdcmd_nonewline = settings.get_string("terminal-change-directory-command") \
                 % GLib.shell_quote(self._path)
             cdcmd = "%s\n" % cdcmd_nonewline
-            self.term.feed_child(cdcmd, len(cdcmd))
-            
+            self.feed_child(cdcmd)
+
             # Restore user input
             restorelinekeys = settings.get_string("terminal-restore-line").decode('string_escape')
-            self.term.feed_child(restorelinekeys, len(restorelinekeys))
+            self.feed_child(restorelinekeys)
 
     def get_widget(self):
         """Return the top-level widget of Nemo Terminal."""
@@ -269,22 +268,22 @@ class NemoTerminal(object):
 #         print "resize term"
 #         height = self.swin.get_allocated_height()
 #         width = self.swin.get_allocated_width()
-#         
+#
 #         # bail if nothing to do
 #         if self.term.get_allocated_height() == height and \
 #             self.term.get_allocated_width() == width:
 #             return
-#         
+#
 #         char_width = self.term.get_char_width()
 #         char_height = self.term.get_char_height()
-#         
+#
 #         inner_border = self.term.get_style().get_property("inner-border")
-#         new_grid_width = width - (inner_border.left + inner_border.right) / char_width 
+#         new_grid_width = width - (inner_border.left + inner_border.right) / char_width
 #         new_grid_height = height - (inner_border.top + inner_border.bottom) / char_height
-#         
+#
 #         grid_width = new_grid_width if new_grid_width > 0 else 0
 #         grid_height = new_grid_height if new_grid_height > 0 else 0
-# 
+#
 #         self.term.set_size(grid_width, grid_height)
 
     def set_visible(self, visible):
@@ -311,7 +310,7 @@ class NemoTerminal(object):
         about_dlg.set_copyright("Nemo Terminal is based heavily on Nautilus Terminal by %s Copyright (c) 2011" % __author__)
         about_dlg.set_authors([__author__, __maintainer__])
         logo = Gtk.Image.new_from_file(
-                "/usr/share/nemo-terminal/logo_120x120.png")
+            "/usr/share/nemo-terminal/logo_120x120.png")
         about_dlg.set_logo(logo.get_pixbuf())
         #Signal
         about_dlg.connect("response", lambda w, r: w.destroy())
@@ -337,26 +336,16 @@ class NemoTerminal(object):
 
     def _shell_is_busy(self):
         """Check if the shell is waiting for a command or not."""
-        wchan_path = "/proc/%i/wchan" % self.shell_pid
-        wchan = open(wchan_path, "r").read()
-        if wchan == "n_tty_read":
+
+        pty = self.term.get_pty()
+        fd = pty.get_fd()
+
+        fgpid = os.tcgetpgrp(fd)
+
+        if fgpid == -1 or fgpid == self.shell_pid:
             return False
-        elif wchan == "schedule":
-            shell_stack_path = "/proc/%i/stack" % self.shell_pid
-            try:
-                for line in open(shell_stack_path, "r"):
-                    if line.split(" ")[-1].startswith("n_tty_read"):
-                        return False
-                return True
-            except IOError:
-                #We can't know...
-                return False
-        elif wchan == "wait_woken":
-            return False
-        elif wchan == "do_wait":
-            return True
-        else:
-            return True
+
+        return True
 
     def _uri_to_path(self, uri):
         """Returns the path corresponding of the given URI.
@@ -364,7 +353,12 @@ class NemoTerminal(object):
         Args:
             uri -- The URI to convert."""
         gfile = Gio.file_parse_name(uri)
-        return gfile.get_path()
+
+        path = gfile.get_path()
+        if path:
+            return path
+        else:
+            return ""
 
     def _set_term_height(self, height):
         """Change the terminal height.
@@ -373,40 +367,51 @@ class NemoTerminal(object):
             height -- The new height (in lines).
         """
         self.hbox.set_size_request(-1,
-                height * self.term.get_char_height() + 2)
+                                   height * self.term.get_char_height() + 2)
 
     def _on_term_popup_menu(self, widget, event=None):
         """Displays the contextual menu on right-click and menu-key."""
-        if event: #button-release-event
-            if event.type == Gdk.EventType.BUTTON_RELEASE \
-            and event.button != 3:
-                return
+        if event and event.button != 3:
+            return Gdk.EVENT_PROPAGATE
         # Should the Paste Filenames option be displayed?
         gtkClipboard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
         if gtkClipboard.wait_is_uris_available():
             self.menu_item_pastefilenames.show()
         else:
             self.menu_item_pastefilenames.hide()
-        self.menu.popup(None, None, None, None, 3, 0)
+        self.menu.popup_at_pointer(event)
 
-    def _on_term_child_exited(self, term):
+        return Gdk.EVENT_STOP
+
+    def _on_term_child_exited(self, term, status):
         """Called when the shell is terminated.
 
         Args:
             term -- The VTE terminal (self.term).
         """
         if not self._respawn_lock:
-            if VTE_2_90_API:
-                self.shell_pid = self.term.fork_command_full(Vte.PtyFlags.DEFAULT, self._path, [terminal_or_default()], None, GLib.SpawnFlags.SEARCH_PATH, None, None)[1]
-            else:
-                self.shell_pid = self.term.spawn_sync(Vte.PtyFlags.DEFAULT, self._path, [terminal_or_default()], None, GLib.SpawnFlags.SEARCH_PATH, None, None, None)[1]
+            if self._path == "":
+                self._path = GLib.get_home_dir()
+            self.shell_pid = self.term.spawn_sync(Vte.PtyFlags.DEFAULT, self._path, [terminal_or_default()], None, GLib.SpawnFlags.SEARCH_PATH, None, None, None)[1]
 
     def _on_drag_data_received(self, widget, drag_context, x, y, data, info, time):
         """Handles drag & drop."""
         for uri in data.get_uris():
-            path = "'%s' " % self._uri_to_path(uri).replace("'", r"'\''")
-            self.term.feed_child(path, len(path))
+            if self._uri_to_path(uri) == "":
+                continue
 
+            path = "'%s' " % self._uri_to_path(uri).replace("'", r"'\''")
+            self.feed_child(path)
+
+    def feed_child(self, text):
+        """
+        gobject-introspection/python-gi differences force us to try with and
+        without the text length. One of them will work.
+        """
+        try:
+            self.term.feed_child(text)
+        except TypeError:
+            self.term.feed_child(text, len(text))
 
 class Crowbar(object):
     """Modify the Nemo' widget tree when the crowbar is inserted in it.
@@ -511,21 +516,20 @@ class Crowbar(object):
                     hbox = child
             if not hbox:
                 print("[%s] W: Can't find the GtkHBox..."
-                        % __app_disp_name__)
+                      % __app_disp_name__)
                 return
             if not hasattr(hbox, "nt"):
                 print("[%s] W: Can't find the Nemo Terminal instance..."
-                        % __app_disp_name__)
+                      % __app_disp_name__)
             hbox.nt.destroy()
 
-
-class NemoTerminalProvider(GObject.GObject, Nemo.LocationWidgetProvider):
+class NemoTerminalProvider(GObject.GObject, Nemo.LocationWidgetProvider, Nemo.NameAndDescProvider):
     """Provides Nemo Terminal in Nemo."""
 
     def __init__(self):
         """The constructor."""
         print("[%s] I: Initializing the Nemo extension"
-                % __app_disp_name__)
+              % __app_disp_name__)
 
     def get_widget(self, uri, window):
         """Returns a "crowbar" that will add a terminal in Nemo.
@@ -557,15 +561,21 @@ class NemoTerminalProvider(GObject.GObject, Nemo.LocationWidgetProvider):
             event -- The detail of the event.
         """
         if event.keyval == \
-            Gdk.keyval_from_name(settings.get_string("terminal-hotkey")):
+                Gdk.keyval_from_name(settings.get_string("terminal-hotkey")):
             window.term_visible = not window.term_visible
+            settings.set_boolean("default-visible",  window.term_visible)
             for callback in window.toggle_hide_cb:
                 callback(window.term_visible)
-            return True #Stop the event propagation
+            return Gdk.EVENT_STOP #Stop the event propagation
+
+        return Gdk.EVENT_PROPAGATE
+
+    def get_name_and_desc(self):
+        return [("Nemo Terminal:::Embedded terminal for Nemo")]
 
 if __name__ == "__main__":
     #Code for testing Nemo Terminal outside of Nemo
-    print("%s %s\nOriginally By %s\nPorted for Nemo by: %s" 
+    print("%s %s\nOriginally By %s\nPorted for Nemo by: %s"
           % (__app_disp_name__, __version__, __author__, __maintainer__))
     win = Gtk.Window()
     win.set_title("%s %s" % (__app_disp_name__, __version__))
@@ -573,7 +583,7 @@ if __name__ == "__main__":
     nterm._respawn_lock = True
     nterm.term.connect("child-exited", Gtk.main_quit)
     nterm.get_widget().set_size_request(nterm.term.get_char_width() * 80 + 2,
-            nterm.term.get_char_height() * 24 + 2)
+                                        nterm.term.get_char_height() * 24 + 2)
     win.connect_after("destroy", Gtk.main_quit)
     win.add(nterm.get_widget())
     win.show_all()
